@@ -7,90 +7,111 @@ mod serialize;
 mod tests;
 
 use std::path::{Path, PathBuf};
-use std::sync::Arc;
+
+#[cfg(feature = "kurbo")]
+use crate::error::ConvertContourError;
 
 #[cfg(feature = "druid")]
 use druid::{Data, Lens};
 
-use crate::error::{Error, ErrorKind, GlifError, GlifErrorInternal};
+use crate::error::{ErrorKind, GlifLoadError, GlifWriteError, StoreError};
+use crate::name::Name;
 use crate::names::NameList;
 use crate::shared_types::PUBLIC_OBJECT_LIBS_KEY;
 use crate::{Color, Guideline, Identifier, Line, Plist, WriteOptions};
 
-/// The name of a glyph.
-pub type GlyphName = Arc<str>;
-
 /// A glyph, loaded from a [`.glif` file][glif].
 ///
+/// Norad can load glif version 1.0 and 2.0, and can save 2.0 only.
+///
 /// [glif]: http://unifiedfontobject.org/versions/ufo3/glyphs/glif/
 #[derive(Debug, Clone, PartialEq)]
 #[cfg_attr(feature = "druid", derive(Lens))]
 pub struct Glyph {
-    pub name: GlyphName,
-    pub format: GlifVersion,
-    pub height: f32,
-    pub width: f32,
+    /// The name of the glyph.
+    pub name: Name,
+    /// Glyph height.
+    pub height: f64,
+    /// Glyph width.
+    pub width: f64,
+    /// A collection of glyph Unicode code points.
+    ///
+    /// The first entry defines the primary Unicode value for this glyph.
     pub codepoints: Vec<char>,
+    /// Arbitrary glyph note.
     pub note: Option<String>,
+    /// A collection of glyph guidelines.
     pub guidelines: Vec<Guideline>,
+    /// A collection of glyph anchors.
     pub anchors: Vec<Anchor>,
+    /// A collection of glyph components.
     pub components: Vec<Component>,
+    /// A collection of glyph contours.
     pub contours: Vec<Contour>,
+    /// Glyph image data.
     pub image: Option<Image>,
+    /// Glyph library data.
     pub lib: Plist,
 }
 
 impl Glyph {
-    /// Load the glyph at this path.
+    /// Attempt to parse a `Glyph` from a [`.glif`] at the provided path.
     ///
-    /// When loading glyphs in bulk, `load_with_names` should be preferred,
-    /// since it will allow glyph names (in glyphs and components) to be shared
-    /// between instances.
-    pub fn load(path: impl AsRef<Path>) -> Result<Self, Error> {
+    /// [`.glif`]: http://unifiedfontobject.org/versions/ufo3/glyphs/glif/
+    pub fn load(path: impl AsRef<Path>) -> Result<Self, GlifLoadError> {
         let path = path.as_ref();
         let names = NameList::default();
         Glyph::load_with_names(path, &names)
     }
 
-    pub fn load_with_names(path: &Path, names: &NameList) -> Result<Self, Error> {
-        let data = std::fs::read(path)?;
-        parse::GlifParser::from_xml(&data, Some(names)).map_err(|e| match e {
-            GlifErrorInternal::Xml(e) => e.into(),
-            GlifErrorInternal::Spec { kind, position } => {
-                GlifError { kind, position, path: Some(path.to_owned()) }.into()
-            }
-        })
+    /// Attempt to load the glyph at `path`, reusing names from the `NameList`.
+    ///
+    /// This uses string interning to reuse allocations when a glyph name
+    /// occurs multiple times (such as in components or in different layers).
+    pub(crate) fn load_with_names(path: &Path, names: &NameList) -> Result<Self, GlifLoadError> {
+        std::fs::read(path)
+            .map_err(GlifLoadError::Io)
+            .and_then(|data| parse::GlifParser::from_xml(&data, Some(names)))
     }
 
     #[doc(hidden)]
-    pub fn save<P: AsRef<Path>>(&self, path: P) -> Result<(), Error> {
+    pub fn save<P: AsRef<Path>>(&self, path: P) -> Result<(), GlifWriteError> {
         let path = path.as_ref();
         let opts = WriteOptions::default();
         self.save_with_options(path, &opts)
     }
 
-    pub(crate) fn save_with_options(&self, path: &Path, opts: &WriteOptions) -> Result<(), Error> {
-        if self.format != GlifVersion::V2 {
-            return Err(Error::DowngradeUnsupported);
-        }
+    pub(crate) fn save_with_options(
+        &self,
+        path: &Path,
+        opts: &WriteOptions,
+    ) -> Result<(), GlifWriteError> {
         if self.lib.contains_key(PUBLIC_OBJECT_LIBS_KEY) {
-            return Err(Error::PreexistingPublicObjectLibsKey);
+            return Err(GlifWriteError::PreexistingPublicObjectLibsKey);
         }
 
         let data = self.encode_xml_with_options(opts)?;
-        std::fs::write(path, &data)?;
+        std::fs::write(path, &data).map_err(GlifWriteError::Io)?;
+
         Ok(())
     }
 
-    /// Create a new glyph with the given name.
-    pub fn new_named<S: Into<GlyphName>>(name: S) -> Self {
-        Glyph::new(name.into(), GlifVersion::V2)
+    /// Returns a new, "empty" [`Glyph`] with the given `name`.
+    ///
+    /// # Panics
+    ///
+    /// panics if `name` is empty or if it contains any [control characters].
+    ///
+    /// [control characters]: https://unifiedfontobject.org/versions/ufo3/conventions/#controls
+    pub fn new(name: &str) -> Self {
+        Glyph::new_impl(Name::new_raw(name))
     }
 
-    pub(crate) fn new(name: GlyphName, format: GlifVersion) -> Self {
+    // this impl lets the crate pass an explicit `Name`, which is reused between
+    // multiple layers and components
+    pub(crate) fn new_impl(name: Name) -> Self {
         Glyph {
             name,
-            format,
             height: 0.0,
             width: 0.0,
             codepoints: Vec::new(),
@@ -104,48 +125,68 @@ impl Glyph {
         }
     }
 
+    /// Returns true if [`Glyph`] contains one or more [`Component`]s.
+    pub fn has_component(&self) -> bool {
+        !self.components.is_empty()
+    }
+
+    /// Returns the number of [`Component`]s in the Glyph.
+    pub fn component_count(&self) -> usize {
+        self.components.len()
+    }
+
+    /// Returns true if the Glyph contains one or more [`Component`]s with base
+    /// glyph name `basename`.
+    pub fn has_component_with_base(&self, basename: &str) -> bool {
+        self.components.iter().any(|x| *x.base == *basename)
+    }
+
+    /// Returns an iterator over immutable [`Component`] references filtered by base glyph name.
+    pub fn get_components_with_base<'b, 'a: 'b>(
+        &'a self,
+        basename: &'b str,
+    ) -> impl Iterator<Item = &'a Component> + 'b {
+        self.components.iter().filter(move |x| *x.base == *basename)
+    }
+
     /// Move libs from the lib's `public.objectLibs` into the actual objects.
     /// The key will be removed from the glyph lib.
-    fn load_object_libs(&mut self) -> Result<(), ErrorKind> {
+    fn load_object_libs(&mut self) -> Result<(), GlifLoadError> {
+        // Use a macro to reduce boilerplate, to avoid having to mess with the typing system.
+        macro_rules! transfer_lib {
+            ($object:expr, $object_libs:expr) => {
+                if let Some(id) = $object.identifier().map(|v| v.as_str()) {
+                    if let Some(lib) = $object_libs.remove(id) {
+                        let lib = lib
+                            .into_dictionary()
+                            .ok_or(GlifLoadError::ObjectLibMustBeDictionary(id.into()))?;
+                        $object.replace_lib(lib);
+                    }
+                }
+            };
+        }
+
         let mut object_libs = match self.lib.remove(PUBLIC_OBJECT_LIBS_KEY) {
-            Some(lib) => lib.into_dictionary().ok_or(ErrorKind::BadLib)?,
+            Some(lib) => {
+                lib.into_dictionary().ok_or(GlifLoadError::PublicObjectLibsMustBeDictionary)?
+            }
             None => return Ok(()),
         };
 
         for anchor in &mut self.anchors {
-            if let Some(lib) = anchor.identifier().and_then(|id| object_libs.remove(id.as_str())) {
-                let lib = lib.into_dictionary().ok_or(ErrorKind::BadLib)?;
-                anchor.replace_lib(lib);
-            }
+            transfer_lib!(anchor, object_libs);
         }
-
         for guideline in &mut self.guidelines {
-            if let Some(lib) = guideline.identifier().and_then(|id| object_libs.remove(id.as_str()))
-            {
-                let lib = lib.into_dictionary().ok_or(ErrorKind::BadLib)?;
-                guideline.replace_lib(lib);
-            }
+            transfer_lib!(guideline, object_libs);
         }
-
         for contour in &mut self.contours {
-            if let Some(lib) = contour.identifier().and_then(|id| object_libs.remove(id.as_str())) {
-                let lib = lib.into_dictionary().ok_or(ErrorKind::BadLib)?;
-                contour.replace_lib(lib);
-            }
+            transfer_lib!(contour, object_libs);
             for point in &mut contour.points {
-                if let Some(lib) = point.identifier().and_then(|id| object_libs.remove(id.as_str()))
-                {
-                    let lib = lib.into_dictionary().ok_or(ErrorKind::BadLib)?;
-                    point.replace_lib(lib);
-                }
+                transfer_lib!(point, object_libs);
             }
         }
         for component in &mut self.components {
-            if let Some(lib) = component.identifier().and_then(|id| object_libs.remove(id.as_str()))
-            {
-                let lib = lib.into_dictionary().ok_or(ErrorKind::BadLib)?;
-                component.replace_lib(lib);
-            }
+            transfer_lib!(component, object_libs);
         }
 
         Ok(())
@@ -196,7 +237,6 @@ impl Glyph {
 impl Data for Glyph {
     fn same(&self, other: &Glyph) -> bool {
         self.name.same(&other.name)
-            && self.format.same(&other.format)
             && self.height.same(&other.height)
             && self.width.same(&other.width)
             && self.codepoints == other.codepoints
@@ -210,67 +250,68 @@ impl Data for Glyph {
     }
 }
 
-/// Version of a `.glif` file, per the [UFO spec].
-///
-/// [UFO spec]: https://unifiedfontobject.org/versions/ufo1/glyphs/glif/#specification
-#[derive(Debug, Clone, PartialEq)]
-#[cfg_attr(feature = "druid", derive(Data))]
-pub enum GlifVersion {
-    V1 = 1,
-    V2 = 2,
-}
-
-/// An reference position in a glyph, such as for attaching accents.
+/// A reference position in a glyph, such as for attaching accents.
 ///
 /// See the [Anchor section] of the UFO spec for more information.
 ///
 /// [Anchor section]: https://unifiedfontobject.org/versions/ufo3/glyphs/glif/#anchor
 #[derive(Debug, Clone, PartialEq)]
 pub struct Anchor {
-    pub x: f32,
-    pub y: f32,
-    /// An arbitrary name for the anchor.
-    pub name: Option<String>,
+    /// Anchor x coordinate value.
+    pub x: f64,
+    /// Anchor y coordinate value.
+    pub y: f64,
+    /// Optional arbitrary name for the anchor.
+    pub name: Option<Name>,
+    /// Optional anchor color.
     pub color: Option<Color>,
-    /// Unique identifier for the anchor within the glyph. This attribute is only required
-    /// when a lib is present and should otherwise only be added as needed.
+    /// Optional unique identifier for the anchor within the glyph.
+    ///
+    /// This attribute is only required when a lib is present and should otherwise only be added as needed.
     identifier: Option<Identifier>,
-    /// The anchor's lib for arbitary data.
+    /// Optional anchor lib for arbitary data.
     lib: Option<Plist>,
 }
 
 /// A reference to another glyph, to be included in this glyph's outline.
 #[derive(Debug, Clone, PartialEq)]
 pub struct Component {
-    /// The name of the base glyph.
-    pub base: GlyphName,
+    /// The name of the base glyph used in the component.
+    pub base: Name,
+    /// Component affine transormation definition.
     pub transform: AffineTransform,
-    /// Unique identifier for the component within the glyph. This attribute is only required
-    /// when a lib is present and should otherwise only be added as needed.
+    /// Optional unique identifier for the component within the glyph.
+    ///
+    /// This attribute is only required when a lib is present and should otherwise only
+    /// be added as needed.
     identifier: Option<Identifier>,
-    /// The component's lib for arbitary data.
+    ///  Optional lib for arbitary component data.
     lib: Option<Plist>,
 }
 
 /// A single open or closed bezier path segment.
 #[derive(Debug, Clone, Default, PartialEq)]
 pub struct Contour {
+    /// A collection of contour points.
     pub points: Vec<ContourPoint>,
-    /// Unique identifier for the contour within the glyph. This attribute is only required
-    /// when a lib is present and should otherwise only be added as needed.
+    /// Unique identifier for the contour within the glyph.
+    ///
+    /// This attribute is only required when a lib is present and should otherwise only
+    /// be added as needed.
     identifier: Option<Identifier>,
-    /// The contour's lib for arbitary data.
+    /// Optional lib for arbitary contour data.
     lib: Option<Plist>,
 }
 
 impl Contour {
-    fn is_closed(&self) -> bool {
+    /// Whether the contour is closed.
+    pub fn is_closed(&self) -> bool {
         self.points.first().map_or(true, |v| v.typ != PointType::Move)
     }
 
     /// Converts the `Contour` to a [`kurbo::BezPath`].
     #[cfg(feature = "kurbo")]
-    pub fn to_kurbo(&self) -> Result<kurbo::BezPath, Error> {
+    pub fn to_kurbo(&self) -> Result<kurbo::BezPath, ConvertContourError> {
         let mut path = kurbo::BezPath::new();
         let mut offs = std::collections::VecDeque::new();
         let mut points = if self.is_closed() {
@@ -296,10 +337,10 @@ impl Contour {
                 PointType::OffCurve => offs.push_back(kurbo_point),
                 PointType::Curve => {
                     match offs.make_contiguous() {
-                        [] => return Err(Error::ConvertContour(ErrorKind::BadPoint)),
+                        [] => return Err(ConvertContourError::new(ErrorKind::BadPoint)),
                         [p1] => path.quad_to(*p1, kurbo_point),
                         [p1, p2] => path.curve_to(*p1, *p2, kurbo_point),
-                        _ => return Err(Error::ConvertContour(ErrorKind::TooManyOffCurves)),
+                        _ => return Err(ConvertContourError::new(ErrorKind::TooManyOffCurves)),
                     };
                     offs.clear();
                 }
@@ -323,15 +364,21 @@ impl Contour {
 /// A single point in a [`Contour`].
 #[derive(Debug, Clone, PartialEq)]
 pub struct ContourPoint {
-    pub x: f32,
-    pub y: f32,
+    /// Contour point x coordinate value.
+    pub x: f64,
+    /// Contour point y coordinate value.
+    pub y: f64,
+    /// Contour point type.
     pub typ: PointType,
+    /// Whether a smooth curvature should be maintained at this point. Must not be set for off-curve points.
     pub smooth: bool,
-    pub name: Option<String>,
-    /// Unique identifier for the point within the glyph. This attribute is only required
-    /// when a lib is present and should otherwise only be added as needed.
+    /// Optional contour point name.
+    pub name: Option<Name>,
+    /// Optional unique identifier for the point within the glyph.
+    ///
+    /// This attribute is only required when a lib is present and should otherwise only be added as needed.
     identifier: Option<Identifier>,
-    /// The point's lib for arbitary data.
+    /// Optional lib for arbitary contour point data.
     lib: Option<Plist>,
 }
 
@@ -398,19 +445,26 @@ impl std::fmt::Display for PointType {
 #[derive(Debug, Clone, Copy, PartialEq)]
 #[cfg_attr(feature = "druid", derive(Data))]
 pub struct AffineTransform {
-    pub x_scale: f32,
-    pub xy_scale: f32,
-    pub yx_scale: f32,
-    pub y_scale: f32,
-    pub x_offset: f32,
-    pub y_offset: f32,
+    /// x-scale value.
+    pub x_scale: f64,
+    /// xy-scale value.
+    pub xy_scale: f64,
+    /// yx-scale value.
+    pub yx_scale: f64,
+    /// y-scale value.
+    pub y_scale: f64,
+    /// x-offset value.
+    pub x_offset: f64,
+    /// y-offset value.
+    pub y_offset: f64,
 }
 
 impl Anchor {
+    /// Returns a new [`Anchor`] given `x` and `y` coordinate values.
     pub fn new(
-        x: f32,
-        y: f32,
-        name: Option<String>,
+        x: f64,
+        y: f64,
+        name: Option<Name>,
         color: Option<Color>,
         identifier: Option<Identifier>,
         lib: Option<Plist>,
@@ -425,7 +479,7 @@ impl Anchor {
         this
     }
 
-    /// Returns an immutable reference to the anchor's lib.
+    /// Returns a reference to the anchor's lib.
     pub fn lib(&self) -> Option<&Plist> {
         self.lib.as_ref()
     }
@@ -449,7 +503,7 @@ impl Anchor {
         self.lib.take()
     }
 
-    /// Returns an immutable reference to the anchor's identifier.
+    /// Returns a reference to the anchor's identifier.
     pub fn identifier(&self) -> Option<&Identifier> {
         self.identifier.as_ref()
     }
@@ -462,6 +516,7 @@ impl Anchor {
 }
 
 impl Contour {
+    /// Returns a new [`Contour`] given a vector of contour points.
     pub fn new(
         points: Vec<ContourPoint>,
         identifier: Option<Identifier>,
@@ -477,7 +532,7 @@ impl Contour {
         this
     }
 
-    /// Returns an immutable reference to the contour's lib.
+    /// Returns a reference to the contour's lib.
     pub fn lib(&self) -> Option<&Plist> {
         self.lib.as_ref()
     }
@@ -501,7 +556,7 @@ impl Contour {
         self.lib.take()
     }
 
-    /// Returns an immutable reference to the contour's identifier.
+    /// Returns a reference to the contour's identifier.
     pub fn identifier(&self) -> Option<&Identifier> {
         self.identifier.as_ref()
     }
@@ -514,12 +569,14 @@ impl Contour {
 }
 
 impl ContourPoint {
+    /// Returns a new [`ContourPoint`] given an `x` coordinate value,
+    /// `y` coordinate value, point type, and smooth definition.
     pub fn new(
-        x: f32,
-        y: f32,
+        x: f64,
+        y: f64,
         typ: PointType,
         smooth: bool,
-        name: Option<String>,
+        name: Option<Name>,
         identifier: Option<Identifier>,
         lib: Option<Plist>,
     ) -> Self {
@@ -533,7 +590,7 @@ impl ContourPoint {
         this
     }
 
-    /// Returns an immutable reference to the contour's lib.
+    /// Returns a reference to the contour's lib.
     pub fn lib(&self) -> Option<&Plist> {
         self.lib.as_ref()
     }
@@ -557,7 +614,7 @@ impl ContourPoint {
         self.lib.take()
     }
 
-    /// Returns an immutable reference to the contour's identifier.
+    /// Returns a reference to the contour's identifier.
     pub fn identifier(&self) -> Option<&Identifier> {
         self.identifier.as_ref()
     }
@@ -584,8 +641,11 @@ impl ContourPoint {
 }
 
 impl Component {
+    /// Returns a new [`Component`] given a base glyph name and affine transformation definition.
+    ///
+    /// The 'name' argument should be taken from an existing glyph in  the same layer.
     pub fn new(
-        base: GlyphName,
+        base: Name,
         transform: AffineTransform,
         identifier: Option<Identifier>,
         lib: Option<Plist>,
@@ -600,7 +660,7 @@ impl Component {
         this
     }
 
-    /// Returns an immutable reference to the component's lib.
+    /// Returns a reference to the component's lib.
     pub fn lib(&self) -> Option<&Plist> {
         self.lib.as_ref()
     }
@@ -624,7 +684,7 @@ impl Component {
         self.lib.take()
     }
 
-    /// Returns an immutable reference to the component's identifier.
+    /// Returns a reference to the component's identifier.
     pub fn identifier(&self) -> Option<&Identifier> {
         self.identifier.as_ref()
     }
@@ -659,22 +719,50 @@ impl std::default::Default for AffineTransform {
 /// An image included in a glyph.
 #[derive(Debug, Clone, PartialEq)]
 pub struct Image {
-    /// Not an absolute / relative path, but the name of the image file.
-    pub file_name: PathBuf,
+    /// The name of the image file. Must be a base file name, no subdirectories involved.
+    file_name: PathBuf,
+    /// Optional image color.
     pub color: Option<Color>,
+    /// Affine transformation.
     pub transform: AffineTransform,
 }
 
+impl Image {
+    /// Create a new image.
+    pub fn new(
+        file_name: PathBuf,
+        color: Option<Color>,
+        transform: AffineTransform,
+    ) -> Result<Self, StoreError> {
+        // Note: Mostly mirrors [`Self::validate_entry`].
+        if file_name.as_os_str().is_empty() {
+            return Err(StoreError::EmptyPath);
+        }
+        if file_name.is_absolute() {
+            return Err(StoreError::PathIsAbsolute);
+        }
+        if file_name.parent().map_or(false, |p| !p.as_os_str().is_empty()) {
+            return Err(StoreError::Subdir);
+        }
+        Ok(Self { file_name, color, transform })
+    }
+
+    /// Returns the file name of the image.
+    pub fn file_name(&self) -> &Path {
+        self.file_name.as_path()
+    }
+}
+
 #[cfg(feature = "kurbo")]
 impl From<AffineTransform> for kurbo::Affine {
     fn from(src: AffineTransform) -> kurbo::Affine {
         kurbo::Affine::new([
-            src.x_scale as f64,
-            src.xy_scale as f64,
-            src.yx_scale as f64,
-            src.y_scale as f64,
-            src.x_offset as f64,
-            src.y_offset as f64,
+            src.x_scale,
+            src.xy_scale,
+            src.yx_scale,
+            src.y_scale,
+            src.x_offset,
+            src.y_offset,
         ])
     }
 }
@@ -684,12 +772,12 @@ impl From<kurbo::Affine> for AffineTransform {
     fn from(src: kurbo::Affine) -> AffineTransform {
         let coeffs = src.as_coeffs();
         AffineTransform {
-            x_scale: coeffs[0] as f32,
-            xy_scale: coeffs[1] as f32,
-            yx_scale: coeffs[2] as f32,
-            y_scale: coeffs[3] as f32,
-            x_offset: coeffs[4] as f32,
-            y_offset: coeffs[5] as f32,
+            x_scale: coeffs[0],
+            xy_scale: coeffs[1],
+            yx_scale: coeffs[2],
+            y_scale: coeffs[3],
+            x_offset: coeffs[4],
+            y_offset: coeffs[5],
         }
     }
 }
@@ -698,29 +786,26 @@ impl From<kurbo::Affine> for AffineTransform {
 impl From<druid::piet::Color> for Color {
     fn from(src: druid::piet::Color) -> Color {
         let rgba = src.as_rgba_u32();
-        let r = ((rgba >> 24) & 0xff) as f32 / 255.0;
-        let g = ((rgba >> 16) & 0xff) as f32 / 255.0;
-        let b = ((rgba >> 8) & 0xff) as f32 / 255.0;
-        let a = (rgba & 0xff) as f32 / 255.0;
+        let r = ((rgba >> 24) & 0xff) as f64 / 255.0;
+        let g = ((rgba >> 16) & 0xff) as f64 / 255.0;
+        let b = ((rgba >> 8) & 0xff) as f64 / 255.0;
+        let a = (rgba & 0xff) as f64 / 255.0;
         assert!((0.0..=1.0).contains(&b), "b: {}, raw {}", b, (rgba & (0xff << 8)));
 
-        Color {
-            red: r.max(0.0).min(1.0),
-            green: g.max(0.0).min(1.0),
-            blue: b.max(0.0).min(1.0),
-            alpha: a.max(0.0).min(1.0),
-        }
+        Color::new(
+            r.max(0.0).min(1.0),
+            g.max(0.0).min(1.0),
+            b.max(0.0).min(1.0),
+            a.max(0.0).min(1.0),
+        )
+        .unwrap()
     }
 }
 
 #[cfg(feature = "druid")]
 impl From<Color> for druid::piet::Color {
     fn from(src: Color) -> druid::piet::Color {
-        druid::piet::Color::rgba(
-            src.red.into(),
-            src.green.into(),
-            src.blue.into(),
-            src.alpha.into(),
-        )
+        let (red, green, blue, alpha) = src.channels();
+        druid::piet::Color::rgba(red, green, blue, alpha)
     }
 }
