@@ -13,7 +13,7 @@ fn transform() {
 
 #[test]
 fn serialize_empty_glyph() {
-    let glyph = Glyph::new_named("a");
+    let glyph = Glyph::new("a");
     let glif = glyph.encode_xml().unwrap();
     let glif = std::str::from_utf8(&glif).unwrap();
     assert_eq!(
@@ -27,9 +27,33 @@ fn serialize_empty_glyph() {
     );
 }
 
+#[test]
+fn parse_format_minor() {
+    let data = r#"
+ <?xml version="1.0" encoding="UTF-8"?>
+ <glyph name="a" format="2" formatMinor="0">
+ </glyph>
+     "#
+    .trim();
+    // if this doesn't panic life is okay
+    parse_glyph(data.as_bytes()).unwrap();
+}
+
+#[test]
+#[should_panic(expected = "UnsupportedGlifVersion")]
+fn parse_format_unsupported_major() {
+    let data = r#"
+ <?xml version="1.0" encoding="UTF-8"?>
+ <glyph name="a" format="3">
+ </glyph>
+     "#
+    .trim();
+    let _ = parse_glyph(data.as_bytes()).unwrap();
+}
+
 #[test]
 fn serialize_empty_glyph_explicit_line_ending_check() {
-    let glyph = Glyph::new_named("a");
+    let glyph = Glyph::new("a");
     let glif = glyph.encode_xml().unwrap();
     let glif = std::str::from_utf8(&glif).unwrap();
     assert_eq!(
@@ -192,7 +216,6 @@ fn parse_v1_upgrade_anchors() {
             Anchor::new(40.0, 20.0, Some("right".into()), None, None, None),
         ]
     );
-    assert_eq!(glyph.format, GlifVersion::V2);
     assert_eq!(glyph.guidelines, vec![]);
     assert_eq!(glyph.image, None);
     assert_eq!(glyph.lib, Plist::new());
@@ -239,6 +262,132 @@ fn missing_close() {
     let _ = parse_glyph(data.as_bytes()).unwrap();
 }
 
+#[test]
+#[should_panic(expected = "DuplicateElement")]
+fn duplicate_outline() {
+    let data = r#"
+<?xml version="1.0" encoding="UTF-8"?>
+<glyph name="period" format="2">
+  <outline>
+  </outline>
+  <outline/>
+</glyph>
+"#;
+    let _ = parse_glyph(data.as_bytes()).unwrap();
+}
+
+#[test]
+#[should_panic(expected = "ComponentMissingBase")]
+fn component_missing_base() {
+    let data = r#"
+<?xml version="1.0" encoding="UTF-8"?>
+<glyph name="period" format="2">
+  <outline>
+    <component/>
+  </outline>
+</glyph>
+"#;
+    let _ = parse_glyph(data.as_bytes()).unwrap();
+}
+
+#[test]
+#[should_panic(expected = "ComponentEmptyBase")]
+fn component_empty_base() {
+    let data = r#"
+<?xml version="1.0" encoding="UTF-8"?>
+<glyph name="period" format="2">
+  <outline>
+    <component base=""/>
+  </outline>
+</glyph>
+"#;
+    let _ = parse_glyph(data.as_bytes()).unwrap();
+}
+
+#[test]
+#[should_panic(expected = "BadAngle")]
+fn bad_angle() {
+    let data = r#"
+<?xml version="1.0" encoding="UTF-8"?>
+<glyph name="period" format="2">
+  <guideline x="1" y="2" angle="-10"/>
+</glyph>
+"#;
+    let _ = parse_glyph(data.as_bytes()).unwrap();
+}
+
+#[test]
+#[should_panic(expected = "LibMustBeDictionary")]
+fn lib_must_be_dict() {
+    let data = r#"
+<?xml version="1.0" encoding="UTF-8"?>
+<glyph name="period" format="2">
+    <lib>
+        <string>I am a creative professional :)</string>
+    </lib>
+</glyph>
+"#;
+    let _ = parse_glyph(data.as_bytes()).unwrap();
+}
+
+#[test]
+#[should_panic(expected = "PublicObjectLibsMustBeDictionary")]
+fn public_object_libs_must_be_dict() {
+    let data = r#"
+<?xml version="1.0" encoding="UTF-8"?>
+<glyph name="period" format="2">
+    <lib>
+        <dict>
+            <key>public.objectLibs</key>
+            <string>0,1,0,0.5</string>
+        </dict>
+    </lib>
+</glyph>
+"#;
+    let _ = parse_glyph(data.as_bytes()).unwrap();
+}
+
+#[test]
+#[should_panic(expected = "ObjectLibMustBeDictionary")]
+fn object_lib_must_be_dict() {
+    let data = r#"
+<?xml version="1.0" encoding="UTF-8"?>
+<glyph name="period" format="2">
+	<anchor name="top" x="74" y="197" identifier="KN3WZjorob"/>
+    <lib>
+        <dict>
+            <key>public.objectLibs</key>
+            <dict>
+                <key>KN3WZjorob</key>
+                <string>0,1,0,0.5</string>
+            </dict>
+        </dict>
+    </lib>
+</glyph>
+"#;
+    let _ = parse_glyph(data.as_bytes()).unwrap();
+}
+
+#[test]
+fn if_no_one_uses_your_lib_is_it_broken() {
+    let data = r#"
+<?xml version="1.0" encoding="UTF-8"?>
+<glyph name="period" format="2">
+    <lib>
+        <dict>
+            <key>public.objectLibs</key>
+            <dict>
+                <key>KN3WZjorob</key>
+                <string>0,1,0,0.5</string>
+            </dict>
+        </dict>
+    </lib>
+</glyph>
+"#;
+    let glyph = parse_glyph(data.as_bytes()).unwrap();
+    assert!(glyph.lib.get("public.objectLibs").is_none());
+}
+
 #[test]
 fn parse_note() {
     let bytes = include_bytes!("../../testdata/note.glif");
@@ -258,7 +407,6 @@ fn save() {
 
     let glyph2 = parse_glyph(buf.as_slice()).expect("re-parse failed");
     assert_eq!(glyph.name, glyph2.name);
-    assert_eq!(glyph.format, glyph2.format);
     assert_eq!(glyph.height, glyph2.height);
     assert_eq!(glyph.width, glyph2.width);
     assert_eq!(glyph.codepoints, glyph2.codepoints);
@@ -280,7 +428,7 @@ fn notdef_failure() {
 #[test]
 fn druid_from_color() {
     let color = druid::piet::Color::rgba(1.0, 0.11, 0.5, 0.23);
-    let color2: druid::piet::Color = Color { red: 1.0, green: 0.11, blue: 0.5, alpha: 0.23 }.into();
+    let color2: druid::piet::Color = Color::new(1.0, 0.11, 0.5, 0.23).unwrap().into();
     assert_eq!(color2.as_rgba_u32(), color.as_rgba_u32());
 }
 
@@ -513,6 +661,22 @@ fn unexpected_line_after_offcurve2() {
     let _ = parse_glyph(data.as_bytes()).unwrap();
 }
 
+#[test]
+#[should_panic(expected = "InvalidName")]
+fn invalid_name() {
+    let data = "
+        <?xml version=\"1.0\" encoding=\"UTF-8\"?>
+        <glyph name=\"\x01\" format=\"2\">
+            <outline>
+                <contour>
+                    <point x=\"572\" y=\"667\"/>
+                    <point x=\"479\" y=\"714\"/>
+                </contour>
+            </outline>
+        </glyph>
+    ";
+    let _ = parse_glyph(data.as_bytes()).unwrap();
+}
 #[test]
 #[should_panic(expected = "UnexpectedPointAfterOffCurve")]
 fn unexpected_line_after_offcurve3() {
@@ -608,3 +772,99 @@ fn pointtype_from_str_trait() {
 fn pointtype_from_str_unknown_type() {
     PointType::from_str("bogus").unwrap();
 }
+
+#[test]
+fn components_load() {
+    let bytes = include_bytes!("../../testdata/MutatorSansLightWide.ufo/glyphs/A_dieresis.glif");
+    let glyph = parse_glyph(bytes).expect("initial load failed");
+    // component order
+    assert_eq!(glyph.components[0].base, "A");
+    assert_eq!(glyph.components[1].base, "dieresis");
+    let error_margin = f64::EPSILON;
+    // component affine transforms
+    assert!(glyph.components[0].transform.x_scale - 1.0 < error_margin);
+    assert!(glyph.components[0].transform.y_scale - 1.0 < error_margin);
+    assert!(glyph.components[0].transform.xy_scale - 0.0 < error_margin);
+    assert!(glyph.components[0].transform.yx_scale - 0.0 < error_margin);
+    assert!(glyph.components[0].transform.x_offset - 0.0 < error_margin);
+    assert!(glyph.components[0].transform.y_offset - 0.0 < error_margin);
+
+    assert!(glyph.components[1].transform.x_scale - 1.0 < error_margin);
+    assert!(glyph.components[1].transform.y_scale - 1.0 < error_margin);
+    assert!(glyph.components[1].transform.xy_scale - 0.0 < error_margin);
+    assert!(glyph.components[1].transform.yx_scale - 0.0 < error_margin);
+    assert!(glyph.components[1].transform.x_offset - 421.0 < error_margin);
+    assert!(glyph.components[1].transform.y_offset - 20.0 < error_margin);
+}
+
+#[test]
+fn has_component() {
+    let bytes = include_bytes!("../../testdata/MutatorSansLightWide.ufo/glyphs/A_dieresis.glif");
+    let glyph = parse_glyph(bytes).expect("initial load failed");
+    assert!(glyph.has_component());
+
+    let bytes = include_bytes!("../../testdata/MutatorSansLightWide.ufo/glyphs/A_.glif");
+    let glyph = parse_glyph(bytes).expect("initial load failed");
+    assert!(!glyph.has_component());
+}
+
+#[test]
+fn component_count() {
+    let bytes = include_bytes!("../../testdata/MutatorSansLightWide.ufo/glyphs/A_dieresis.glif");
+    let glyph = parse_glyph(bytes).expect("initial load failed");
+    assert_eq!(glyph.component_count(), 2);
+
+    let bytes = include_bytes!("../../testdata/MutatorSansLightWide.ufo/glyphs/A_.glif");
+    let glyph = parse_glyph(bytes).expect("initial load failed");
+    assert_eq!(glyph.component_count(), 0);
+}
+
+#[test]
+fn get_components_with_base() {
+    let bytes = include_bytes!("../../testdata/MutatorSansLightWide.ufo/glyphs/A_dieresis.glif");
+    let glyph = parse_glyph(bytes).expect("initial load failed");
+
+    assert_eq!(glyph.components[0].base, "A");
+    assert_eq!(glyph.components[1].base, "dieresis");
+
+    let component_a_vec = glyph.get_components_with_base("A").collect::<Vec<&Component>>();
+    assert!(component_a_vec.len() == 1);
+    assert_eq!(glyph.components[0], *component_a_vec[0]);
+
+    let component_dieresis_vec =
+        glyph.get_components_with_base("dieresis").collect::<Vec<&Component>>();
+    assert!(component_dieresis_vec.len() == 1);
+    assert_eq!(glyph.components[1], *component_dieresis_vec[0]);
+}
+
+#[test]
+fn get_components_with_base_multiple_same_base_components() {
+    let bytes = include_bytes!("../../testdata/MutatorSansLightWide.ufo/glyphs/quotedblbase.glif");
+    let glyph = parse_glyph(bytes).expect("initial load failed");
+    let error_margin = f64::EPSILON;
+    assert_eq!(glyph.components[0].base, "comma");
+    assert!(glyph.components[0].transform.x_offset - 0.0 < error_margin);
+    assert_eq!(glyph.components[1].base, "comma");
+    assert!(glyph.components[1].transform.x_offset - 130.0 < error_margin);
+
+    let component_comma_vec = glyph.get_components_with_base("comma").collect::<Vec<&Component>>();
+    assert!(component_comma_vec.len() == 2);
+    assert_eq!(glyph.components[0], *component_comma_vec[0]);
+    assert_eq!(glyph.components[1], *component_comma_vec[1]);
+}
+
+#[test]
+fn get_components_with_base_missing() {
+    let bytes = include_bytes!("../../testdata/MutatorSansLightWide.ufo/glyphs/A_dieresis.glif");
+    let glyph = parse_glyph(bytes).expect("initial load failed");
+    assert!(glyph.get_components_with_base("Z").next().is_none());
+}
+
+#[test]
+fn has_component_with_base() {
+    let bytes = include_bytes!("../../testdata/MutatorSansLightWide.ufo/glyphs/A_dieresis.glif");
+    let glyph = parse_glyph(bytes).expect("initial load failed");
+    assert!(glyph.has_component_with_base("A"));
+    assert!(glyph.has_component_with_base("dieresis"));
+    assert!(!glyph.has_component_with_base("Z"));
+}
