@@ -1,12 +1,13 @@
 use std::collections::{BTreeMap, HashMap, HashSet};
 use std::path::Path;
 
+use crate::error::FontLoadError;
+use crate::font::LIB_FILE;
 use crate::fontinfo::FontInfo;
 use crate::groups::Groups;
 use crate::kerning::Kerning;
 use crate::names::NameList;
-use crate::shared_types::IntegerOrFloat;
-use crate::Error;
+use crate::Name;
 
 /// Convert kerning groups and pairs from v1 and v2 informal conventions to
 /// v3 formal conventions. Converted groups are added (duplicated) rather than
@@ -27,17 +28,17 @@ pub(crate) fn upconvert_kerning(
     // Make lists of groups referenced in kerning pairs, based on their side.
     for (first, seconds) in kerning {
         if groups.contains_key(first)
-            && !glyph_set.contains(first.as_str())
+            && !glyph_set.contains(&first)
             && !first.starts_with("public.kern1.")
         {
-            groups_first.insert(first.to_string());
+            groups_first.insert(first.clone());
         }
         for second in seconds.keys() {
             if groups.contains_key(second)
-                && !glyph_set.contains(second.as_str())
+                && !glyph_set.contains(&second)
                 && !second.starts_with("public.kern2.")
             {
-                groups_second.insert(second.to_string());
+                groups_second.insert(second.clone());
             }
         }
     }
@@ -45,22 +46,22 @@ pub(crate) fn upconvert_kerning(
     // Duplicate kerning groups with a new name.
     let mut groups_new = groups.clone();
 
-    let mut groups_first_old_to_new: HashMap<&String, String> = HashMap::new();
+    let mut groups_first_old_to_new: HashMap<Name, Name> = HashMap::new();
     for first in &groups_first {
         let first_new = make_unique_group_name(
-            format!("public.kern1.{}", first.replace("@MMK_L_", "")),
+            Name::new(&format!("public.kern1.{}", first.replace("@MMK_L_", ""))).unwrap(),
             &groups_new,
         );
-        groups_first_old_to_new.insert(first, first_new.to_string());
+        groups_first_old_to_new.insert(first.clone(), first_new.clone());
         groups_new.insert(first_new, groups_new.get(first).unwrap().clone());
     }
-    let mut groups_second_old_to_new: HashMap<&String, String> = HashMap::new();
+    let mut groups_second_old_to_new: HashMap<Name, Name> = HashMap::new();
     for second in &groups_second {
         let second_new = make_unique_group_name(
-            format!("public.kern2.{}", second.replace("@MMK_R_", "")),
+            Name::new(&format!("public.kern2.{}", second.replace("@MMK_R_", ""))).unwrap(),
             &groups_new,
         );
-        groups_second_old_to_new.insert(second, second_new.to_string());
+        groups_second_old_to_new.insert(second.clone(), second_new.clone());
         groups_new.insert(second_new, groups_new.get(second).unwrap().clone());
     }
 
@@ -69,41 +70,41 @@ pub(crate) fn upconvert_kerning(
 
     for (first, seconds) in kerning {
         let first_new = groups_first_old_to_new.get(first).unwrap_or(first);
-        let mut seconds_new: BTreeMap<String, f32> = BTreeMap::new();
+        let mut seconds_new: BTreeMap<Name, f64> = BTreeMap::new();
         for (second, value) in seconds {
             let second_new = groups_second_old_to_new.get(second).unwrap_or(second);
-            seconds_new.insert(second_new.to_string(), *value);
+            seconds_new.insert(second_new.clone(), *value);
         }
-        kerning_new.insert(first_new.to_string(), seconds_new);
+        kerning_new.insert(first_new.clone(), seconds_new);
     }
 
     (groups_new, kerning_new)
 }
 
-fn make_unique_group_name(name: String, existing_groups: &Groups) -> String {
+fn make_unique_group_name(name: Name, existing_groups: &Groups) -> Name {
     if !existing_groups.contains_key(&name) {
         return name;
     }
 
     let mut counter = 1;
-    let mut new_name = name.to_string();
+    let mut new_name = name.clone();
     while existing_groups.contains_key(&new_name) {
-        new_name = format!("{}{}", name, counter);
+        new_name = Name::new(&format!("{}{}", name, counter)).unwrap();
         counter += 1;
     }
 
     new_name
 }
 
-fn find_known_kerning_groups(groups: &Groups) -> (HashSet<String>, HashSet<String>) {
-    let mut groups_first: HashSet<String> = HashSet::new();
-    let mut groups_second: HashSet<String> = HashSet::new();
+fn find_known_kerning_groups(groups: &Groups) -> (HashSet<Name>, HashSet<Name>) {
+    let mut groups_first: HashSet<Name> = HashSet::new();
+    let mut groups_second: HashSet<Name> = HashSet::new();
 
     for name in groups.keys() {
         if name.starts_with("@MMK_L_") {
-            groups_first.insert(name.to_string());
+            groups_first.insert(name.clone());
         } else if name.starts_with("@MMK_R_") {
-            groups_second.insert(name.to_string());
+            groups_second.insert(name.clone());
         }
     }
 
@@ -118,8 +119,8 @@ fn find_known_kerning_groups(groups: &Groups) -> (HashSet<String>, HashSet<Strin
 pub(crate) fn upconvert_ufov1_robofab_data(
     lib_path: &Path,
     lib: &mut plist::Dictionary,
-    fontinfo: &mut FontInfo,
-) -> Result<Option<String>, Error> {
+    font_info: &mut FontInfo,
+) -> Result<Option<String>, FontLoadError> {
     #[derive(Debug, Deserialize)]
     struct LibData {
         #[serde(rename = "org.robofab.postScriptHintData")]
@@ -136,20 +137,21 @@ pub(crate) fn upconvert_ufov1_robofab_data(
     #[derive(Debug, Deserialize)]
     #[serde(rename_all = "camelCase")]
     struct PsHintingData {
-        blue_fuzz: Option<IntegerOrFloat>,
+        blue_fuzz: Option<f64>,
         blue_scale: Option<f64>,
-        blue_shift: Option<IntegerOrFloat>,
-        blue_values: Option<Vec<Vec<IntegerOrFloat>>>,
-        family_blues: Option<Vec<Vec<IntegerOrFloat>>>,
-        family_other_blues: Option<Vec<Vec<IntegerOrFloat>>>,
+        blue_shift: Option<f64>,
+        blue_values: Option<Vec<Vec<f64>>>,
+        family_blues: Option<Vec<Vec<f64>>>,
+        family_other_blues: Option<Vec<Vec<f64>>>,
         force_bold: Option<bool>,
-        other_blues: Option<Vec<Vec<IntegerOrFloat>>>,
-        h_stems: Option<Vec<IntegerOrFloat>>,
-        v_stems: Option<Vec<IntegerOrFloat>>,
+        other_blues: Option<Vec<Vec<f64>>>,
+        h_stems: Option<Vec<f64>>,
+        v_stems: Option<Vec<f64>>,
     }
 
     // Read lib.plist again because it is easier than pulling out the data manually.
-    let lib_data: LibData = plist::from_file(lib_path)?;
+    let lib_data: LibData = plist::from_file(lib_path)
+        .map_err(|source| FontLoadError::ParsePlist { name: LIB_FILE, source })?;
 
     // Convert features.
     let mut features = String::new();
@@ -177,27 +179,27 @@ pub(crate) fn upconvert_ufov1_robofab_data(
 
     // Convert PostScript hinting data.
     if let Some(ps_hinting_data) = lib_data.ps_hinting_data {
-        fontinfo.postscript_blue_fuzz = ps_hinting_data.blue_fuzz;
-        fontinfo.postscript_blue_scale = ps_hinting_data.blue_scale;
-        fontinfo.postscript_blue_shift = ps_hinting_data.blue_shift;
+        font_info.postscript_blue_fuzz = ps_hinting_data.blue_fuzz;
+        font_info.postscript_blue_scale = ps_hinting_data.blue_scale;
+        font_info.postscript_blue_shift = ps_hinting_data.blue_shift;
         if let Some(blue_values) = ps_hinting_data.blue_values {
-            fontinfo.postscript_blue_values = Some(blue_values.into_iter().flatten().collect());
+            font_info.postscript_blue_values = Some(blue_values.into_iter().flatten().collect());
         };
         if let Some(other_blues) = ps_hinting_data.other_blues {
-            fontinfo.postscript_other_blues = Some(other_blues.into_iter().flatten().collect());
+            font_info.postscript_other_blues = Some(other_blues.into_iter().flatten().collect());
         };
         if let Some(family_blues) = ps_hinting_data.family_blues {
-            fontinfo.postscript_family_blues = Some(family_blues.into_iter().flatten().collect());
+            font_info.postscript_family_blues = Some(family_blues.into_iter().flatten().collect());
         };
         if let Some(family_other_blues) = ps_hinting_data.family_other_blues {
-            fontinfo.postscript_family_other_blues =
+            font_info.postscript_family_other_blues =
                 Some(family_other_blues.into_iter().flatten().collect());
         };
-        fontinfo.postscript_force_bold = ps_hinting_data.force_bold;
-        fontinfo.postscript_stem_snap_h = ps_hinting_data.h_stems;
-        fontinfo.postscript_stem_snap_v = ps_hinting_data.v_stems;
+        font_info.postscript_force_bold = ps_hinting_data.force_bold;
+        font_info.postscript_stem_snap_h = ps_hinting_data.h_stems;
+        font_info.postscript_stem_snap_v = ps_hinting_data.v_stems;
 
-        fontinfo.validate().map_err(|_| Error::FontInfoUpconversion)?;
+        font_info.validate().map_err(FontLoadError::FontInfoV1Upconversion)?;
     }
 
     lib.remove("org.robofab.postScriptHintData");
@@ -217,10 +219,20 @@ mod tests {
     extern crate maplit;
 
     use super::*;
-    use crate::font::{Font, FormatVersion};
-    use crate::glyph::GlyphName;
+    use crate::{
+        font::{Font, FormatVersion},
+        Name,
+    };
     use maplit::btreemap;
 
+    // we don't want this in the crate because it can fail, but it is useful
+    // for creating test data.
+    impl<'a> From<&'a str> for Name {
+        fn from(src: &str) -> Self {
+            Name::new_raw(src)
+        }
+    }
+
     #[test]
     fn test_upconvert_kerning_just_groups() {
         let groups: Groups = btreemap! {
@@ -238,7 +250,8 @@ mod tests {
         let kerning: Kerning = Kerning::new();
         let glyph_set: NameList = ["a", "b", "c", "d", "e", "f", "g", "h", "i", "j"]
             .iter()
-            .map(|s| GlyphName::from(*s))
+            .cloned()
+            .map(Name::from)
             .collect();
 
         let (groups_new, kerning_new) = upconvert_kerning(&groups, &kerning, &glyph_set);
@@ -499,9 +512,9 @@ mod tests {
 
         assert_eq!(ufo_v1.meta.format_version, FormatVersion::V3);
         assert_eq!(ufo_v2.meta.format_version, FormatVersion::V3);
-        assert_eq!(ufo_v1.groups.unwrap(), groups_expected);
-        assert_eq!(ufo_v2.groups.unwrap(), groups_expected);
-        assert_eq!(ufo_v1.kerning.unwrap(), kerning_expected);
-        assert_eq!(ufo_v2.kerning.unwrap(), kerning_expected);
+        assert_eq!(ufo_v1.groups, groups_expected);
+        assert_eq!(ufo_v2.groups, groups_expected);
+        assert_eq!(ufo_v1.kerning, kerning_expected);
+        assert_eq!(ufo_v2.kerning, kerning_expected);
     }
 }
