@@ -10,8 +10,6 @@ use std::os::windows::prelude::*;
 
 use plist::XmlWriteOptions;
 
-use crate::Error;
-
 /// Options that can be set when writing the UFO to disk.
 ///
 /// You construct `WriteOptions` using builder semantics:
@@ -64,8 +62,8 @@ impl WriteOptions {
     /// not contain multiple different characters. As an example, "\t\t" is
     /// fine, but "\t  \t" is not, because it contains both tabs and spaces.
     ///
-    /// This is not good API, but is a work around for the fact that the quick-xml
-    /// and plist crates both represent whitespace in different ways.
+    // This is not good API, but is a work around for the fact that the quick-xml
+    // and plist crates both represent whitespace in different ways.
     ///
     /// # Panics
     ///
@@ -109,35 +107,23 @@ pub enum QuoteChar {
     Double,
 }
 
-/// Write a `plist::Value` to file, providing custom options.
-pub fn write_plist_value_to_file(
-    path: &Path,
-    value: &plist::Value,
-    options: &WriteOptions,
-) -> Result<(), Error> {
-    let mut file = File::create(path)?;
-    let writer = BufWriter::new(&mut file);
-    value.to_writer_xml_with_options(writer, options.xml_options())?;
-    write_quote_style(&file, options)?;
-    file.sync_all()?;
-    Ok(())
-}
-
 /// Write any `Serialize` to file, providing custom options.
-pub fn write_xml_to_file(
+pub(crate) fn write_xml_to_file(
     path: &Path,
     value: &impl serde::Serialize,
     options: &WriteOptions,
-) -> Result<(), Error> {
-    let mut file = File::create(path)?;
+) -> Result<(), CustomSerializationError> {
+    let mut file = File::create(path).map_err(CustomSerializationError::CreateFile)?;
     let buf_writer = BufWriter::new(&mut file);
-    plist::to_writer_xml_with_options(buf_writer, value, options.xml_options())?;
-    write_quote_style(&file, options)?;
-    file.sync_all()?;
+    plist::to_writer_xml_with_options(buf_writer, value, options.xml_options())
+        .map_err(CustomSerializationError::SerializePlist)?;
+    write_quote_style(&file, options).map_err(CustomSerializationError::WriteQuotes)?;
+    file.sync_all().map_err(CustomSerializationError::Sync)?;
     Ok(())
 }
 
-pub fn write_quote_style(file: &File, options: &WriteOptions) -> Result<(), Error> {
+/// Write XML declarations with custom quote formatting options.
+fn write_quote_style(file: &File, options: &WriteOptions) -> Result<(), std::io::Error> {
     // Optionally modify the XML declaration quote style
     match options.quote_style {
         QuoteChar::Single => {
@@ -153,6 +139,19 @@ pub fn write_quote_style(file: &File, options: &WriteOptions) -> Result<(), Erro
     Ok(())
 }
 
+#[derive(Debug, thiserror::Error)]
+#[non_exhaustive]
+pub enum CustomSerializationError {
+    #[error("failed to create file")]
+    CreateFile(#[source] std::io::Error),
+    #[error("failed to serialize Plist")]
+    SerializePlist(#[source] plist::Error),
+    #[error("failed to rewrite quote style")]
+    WriteQuotes(#[source] std::io::Error),
+    #[error("failed to sync file to disk")]
+    Sync(#[source] std::io::Error),
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -161,136 +160,128 @@ mod tests {
     use tempdir::TempDir;
 
     #[test]
-    fn write_lib_plist_default() -> Result<(), Error> {
+    fn write_lib_plist_default() {
         let opt = WriteOptions::default();
         let plist_read = Value::from_file("testdata/MutatorSansLightWide.ufo/lib.plist")
             .expect("failed to read plist");
-        let tmp = TempDir::new("test")?;
+        let tmp = TempDir::new("test").unwrap();
         let filepath = tmp.path().join("lib.plist");
-        write_plist_value_to_file(&filepath, &plist_read, &opt)?;
-        let plist_write = fs::read_to_string(filepath)?;
+        write_xml_to_file(&filepath, &plist_read, &opt).unwrap();
+        let plist_write = fs::read_to_string(filepath).unwrap();
         let str_list = plist_write.split('\n').collect::<Vec<&str>>();
         assert_eq!(str_list[0], "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"); // default uses double quotes
         assert_eq!(str_list[3], "<dict>"); // no space char at first dict tag
         assert_eq!(str_list[4], "\t<key>com.defcon.sortDescriptor</key>"); // single tab spacing by default
         assert_eq!(str_list[6], "\t\t<dict>"); // second level should use two tab char
-        tmp.close()?;
-        Ok(())
+        tmp.close().unwrap();
     }
 
     #[test]
-    fn write_lib_plist_with_custom_whitespace() -> Result<(), Error> {
+    fn write_lib_plist_with_custom_whitespace() {
         let opt = WriteOptions::default().whitespace("  ");
         let plist_read = Value::from_file("testdata/MutatorSansLightWide.ufo/lib.plist")
             .expect("failed to read plist");
-        let tmp = TempDir::new("test")?;
+        let tmp = TempDir::new("test").unwrap();
         let filepath = tmp.path().join("lib.plist");
-        write_plist_value_to_file(&filepath, &plist_read, &opt)?;
-        let plist_write = fs::read_to_string(filepath)?;
+        write_xml_to_file(&filepath, &plist_read, &opt).unwrap();
+        let plist_write = fs::read_to_string(filepath).unwrap();
         let str_list = plist_write.split('\n').collect::<Vec<&str>>();
         assert_eq!(str_list[0], "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"); // default uses double quotes
         assert_eq!(str_list[3], "<dict>"); // no space char at first dict tag
         assert_eq!(str_list[4], "  <key>com.defcon.sortDescriptor</key>"); // should use two space char
         assert_eq!(str_list[6], "    <dict>"); // second level should use four space char
-        tmp.close()?;
-        Ok(())
+        tmp.close().unwrap();
     }
 
     #[test]
-    fn write_lib_plist_with_custom_whitespace_and_single_quotes() -> Result<(), Error> {
+    fn write_lib_plist_with_custom_whitespace_and_single_quotes() {
         let opt = WriteOptions::default().whitespace("  ").quote_char(QuoteChar::Single);
         let plist_read = Value::from_file("testdata/MutatorSansLightWide.ufo/lib.plist")
             .expect("failed to read plist");
-        let tmp = TempDir::new("test")?;
+        let tmp = TempDir::new("test").unwrap();
         let filepath = tmp.path().join("lib.plist");
-        write_plist_value_to_file(&filepath, &plist_read, &opt)?;
-        let plist_write = fs::read_to_string(filepath)?;
+        write_xml_to_file(&filepath, &plist_read, &opt).unwrap();
+        let plist_write = fs::read_to_string(filepath).unwrap();
         let str_list = plist_write.split('\n').collect::<Vec<&str>>();
         assert_eq!(str_list[0], "<?xml version='1.0' encoding='UTF-8'?>"); // should use single quotes
         assert_eq!(str_list[3], "<dict>"); // no space char at first dict tag
         assert_eq!(str_list[4], "  <key>com.defcon.sortDescriptor</key>"); // should use two space char
         assert_eq!(str_list[6], "    <dict>"); // second level should use four space char
-        tmp.close()?;
-        Ok(())
+        tmp.close().unwrap();
     }
 
     #[test]
-    fn write_lib_plist_line_endings() -> Result<(), Error> {
+    fn write_lib_plist_line_endings() {
         let opt = WriteOptions::default();
         let plist_read = Value::from_file("testdata/lineendings/Tester-LineEndings.ufo/lib.plist")
             .expect("failed to read plist");
-        let tmp = TempDir::new("test")?;
+        let tmp = TempDir::new("test").unwrap();
         let filepath = tmp.path().join("lib.plist");
-        write_plist_value_to_file(&filepath, &plist_read, &opt)?;
-        let plist_write = fs::read_to_string(filepath)?;
+        write_xml_to_file(&filepath, &plist_read, &opt).unwrap();
+        let plist_write = fs::read_to_string(filepath).unwrap();
         assert!(plist_write.starts_with("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<!DOCTYPE plist PUBLIC \"-//Apple//DTD PLIST 1.0//EN\" \"http://www.apple.com/DTDs/PropertyList-1.0.dtd\">\n"));
-        tmp.close()?;
-        Ok(())
+        tmp.close().unwrap();
     }
 
     #[test]
-    fn write_fontinfo_plist_default() -> Result<(), Error> {
+    fn write_fontinfo_plist_default() {
         let opt = WriteOptions::default();
         let plist_read = Value::from_file("testdata/MutatorSansLightWide.ufo/fontinfo.plist")
             .expect("failed to read plist");
-        let tmp = TempDir::new("test")?;
+        let tmp = TempDir::new("test").unwrap();
         let filepath = tmp.path().join("fontinfo.plist");
-        write_plist_value_to_file(&filepath, &plist_read, &opt)?;
-        let plist_write = fs::read_to_string(filepath)?;
+        write_xml_to_file(&filepath, &plist_read, &opt).unwrap();
+        let plist_write = fs::read_to_string(filepath).unwrap();
         let str_list = plist_write.split('\n').collect::<Vec<&str>>();
         assert_eq!(str_list[0], "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"); // default uses double quotes
         assert_eq!(str_list[3], "<dict>"); // no space char at first dict tag
         assert_eq!(str_list[4], "\t<key>ascender</key>"); // single tab level spacing by default
-        tmp.close()?;
-        Ok(())
+        tmp.close().unwrap();
     }
 
     #[test]
-    fn write_fontinfo_plist_with_custom_whitespace() -> Result<(), Error> {
+    fn write_fontinfo_plist_with_custom_whitespace() {
         let opt = WriteOptions::default().whitespace("  ");
         let plist_read = Value::from_file("testdata/MutatorSansLightWide.ufo/fontinfo.plist")
             .expect("failed to read plist");
-        let tmp = TempDir::new("test")?;
+        let tmp = TempDir::new("test").unwrap();
         let filepath = tmp.path().join("fontinfo.plist");
-        write_plist_value_to_file(&filepath, &plist_read, &opt)?;
-        let plist_write = fs::read_to_string(filepath)?;
+        write_xml_to_file(&filepath, &plist_read, &opt).unwrap();
+        let plist_write = fs::read_to_string(filepath).unwrap();
         let str_list = plist_write.split('\n').collect::<Vec<&str>>();
         assert_eq!(str_list[0], "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"); // default uses double quotes
         assert_eq!(str_list[3], "<dict>"); // no space char at first dict tag
         assert_eq!(str_list[4], "  <key>ascender</key>"); // should use two space char
-        tmp.close()?;
-        Ok(())
+        tmp.close().unwrap();
     }
 
     #[test]
-    fn write_fontinfo_plist_with_custom_whitespace_and_single_quotes() -> Result<(), Error> {
+    fn write_fontinfo_plist_with_custom_whitespace_and_single_quotes() {
         let opt = WriteOptions::default().whitespace("  ").quote_char(QuoteChar::Single);
         let plist_read = Value::from_file("testdata/MutatorSansLightWide.ufo/fontinfo.plist")
             .expect("failed to read plist");
-        let tmp = TempDir::new("test")?;
+        let tmp = TempDir::new("test").unwrap();
         let filepath = tmp.path().join("fontinfo.plist");
-        write_plist_value_to_file(&filepath, &plist_read, &opt)?;
-        let plist_write = fs::read_to_string(filepath)?;
+        write_xml_to_file(&filepath, &plist_read, &opt).unwrap();
+        let plist_write = fs::read_to_string(filepath).unwrap();
         let str_list = plist_write.split('\n').collect::<Vec<&str>>();
         assert_eq!(str_list[0], "<?xml version='1.0' encoding='UTF-8'?>"); // should use single quotes
         assert_eq!(str_list[3], "<dict>"); // no space char at first dict tag
         assert_eq!(str_list[4], "  <key>ascender</key>"); // should use two space char
-        tmp.close()?;
-        Ok(())
+        tmp.close().unwrap();
     }
 
     #[test]
-    fn write_fontinfo_plist_line_endings() -> Result<(), Error> {
+    fn write_fontinfo_plist_line_endings() {
         let opt = WriteOptions::default();
         let plist_read =
             Value::from_file("testdata/lineendings/Tester-LineEndings.ufo/fontinfo.plist")
                 .expect("failed to read plist");
-        let tmp = TempDir::new("test")?;
+        let tmp = TempDir::new("test").unwrap();
         let filepath = tmp.path().join("fontinfo.plist");
-        write_plist_value_to_file(&filepath, &plist_read, &opt)?;
-        let plist_write = fs::read_to_string(filepath)?;
+        write_xml_to_file(&filepath, &plist_read, &opt).unwrap();
+        let plist_write = fs::read_to_string(filepath).unwrap();
         assert!(plist_write.starts_with("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<!DOCTYPE plist PUBLIC \"-//Apple//DTD PLIST 1.0//EN\" \"http://www.apple.com/DTDs/PropertyList-1.0.dtd\">\n"));
-        tmp.close()?;
-        Ok(())
+        tmp.close().unwrap();
     }
 }
