@@ -1,6 +1,6 @@
 //! Testing saving files.
 
-use norad::{Font, FormatVersion, Glyph, Identifier, Layer, Plist};
+use norad::{Font, FormatVersion, Glyph, Identifier, Name, Plist};
 use plist::Value;
 
 #[test]
@@ -16,14 +16,14 @@ fn save_default() {
 
     let loaded = Font::load(dir).unwrap();
     assert!(loaded.meta.format_version == FormatVersion::V3);
-    assert!(loaded.meta.creator == "org.linebender.norad");
+    assert!(loaded.meta.creator == Some("org.linebender.norad".into()));
     assert_eq!(loaded.layers.len(), 1);
 }
 
 #[test]
 fn save_new_file() {
     let mut my_ufo = Font::new();
-    let mut my_glyph = Glyph::new_named("A");
+    let mut my_glyph = Glyph::new("A");
     my_glyph.codepoints = vec!['A'];
     my_glyph.note = Some("I did a glyph!".into());
     let mut plist = Plist::new();
@@ -50,9 +50,8 @@ fn save_new_file() {
 #[test]
 fn save_fancy() {
     let mut my_ufo = Font::new();
-    let layer_path = "testdata/MutatorSansLightWide.ufo/glyphs";
-    let layer = Layer::load(layer_path, "foreground".into()).unwrap();
-    *my_ufo.default_layer_mut() = layer;
+    let other_ufo = Font::load("testdata/MutatorSansLightWide.ufo").unwrap();
+    *my_ufo.default_layer_mut() = other_ufo.layers.get("foreground").unwrap().clone();
 
     let dir = tempdir::TempDir::new("Fancy.ufo").unwrap();
     my_ufo.save(&dir).unwrap();
@@ -84,7 +83,7 @@ fn roundtrip_object_libs() {
     let ufo2 = Font::load(&dir).unwrap();
     assert!(!ufo2.lib.contains_key("public.objectLibs"));
 
-    let font_guideline_second = &ufo2.font_info.as_ref().unwrap().guidelines.as_ref().unwrap()[1];
+    let font_guideline_second = &ufo2.font_info.guidelines.as_ref().unwrap()[1];
     assert_eq!(
         font_guideline_second.identifier(),
         Some(&Identifier::new("3f0f37d1-52d6-429c-aff4-3f81aed4abf0").unwrap())
@@ -193,8 +192,7 @@ fn object_libs_reject_existing_key() {
     ufo.lib.remove("public.objectLibs");
 
     let glyph = Glyph {
-        name: "test".into(),
-        format: norad::GlifVersion::V2,
+        name: Name::new("test").unwrap(),
         height: 0.,
         width: 0.,
         anchors: vec![],
